@@ -7,18 +7,20 @@
 #include <sys/fcntl.h>
 #include <sys/select.h>
 #include <sys/socket.h>
+#include <sys/stat.h>
 #include <sys/un.h>
-#include <time.h>
 #include <unistd.h>
 
 #include "cryptIpc.h"
 #include "defines/ipc_values.h"
+#include "defines/msys.h"
 #include "ipc.h"
 #include "ipc/cryptCommunicator.h"
 #include "utils/agentLogger.h"
 #include "utils/db/connection_db.h"
 #include "utils/file_io/fileUtils.h"
 #include "utils/file_io/file_io.h"
+#include "utils/file_io/safefile/check_file_path.h"
 #include "utils/json.h"
 #include "utils/logger.h"
 #include "utils/memory.h"
@@ -128,7 +130,7 @@ char* create_passed_socket_path(const char* requested_path) {
   oidc_ipc_dir      = oidc_strcopy(requested_path);
   if (lastChar(oidc_ipc_dir) == '/') {  // only dir specified
     lastChar(oidc_ipc_dir) = '\0';
-  } else {  // full path including file specified
+  } else {                              // full path including file specified
     char* lastSlash = strrchr(oidc_ipc_dir, '/');
     socket_file     = oidc_strcopy(lastSlash + 1);
     char* tmp       = oidc_strncopy(oidc_ipc_dir, lastSlash - oidc_ipc_dir);
@@ -206,6 +208,13 @@ oidc_error_t ipc_server_init(struct connection* con, const char* group_name,
   strcpy(con->server->sun_path, path);
   secFree(path);
   server_socket_path = con->server->sun_path;
+
+#ifndef ANY_MSYS
+  if (check_socket_path(oidc_ipc_dir, group_name) != OIDC_SUCCESS) {
+    return oidc_errno;
+  }
+#endif
+
   return OIDC_SUCCESS;
 }
 
@@ -235,9 +244,10 @@ char* getServerSocketPath() { return server_socket_path; }
  * @param con, a pointer to the connection struct
  * @return @c 0 on success or an errorcode on failure
  */
-int ipc_bindAndListen(struct connection* con) {
+int ipc_bindAndListen(struct connection* con, const char* group) {
   logger(DEBUG, "binding ipc\n");
   unlink(con->server->sun_path);
+  mode_t previous_mask = umask(group ? 0117 : 0177);
   if (bind(*(con->sock), (struct sockaddr*)con->server,
            sizeof(struct sockaddr_un))) {
     logger(ALERT, "binding stream socket: %m");
@@ -245,6 +255,12 @@ int ipc_bindAndListen(struct connection* con) {
     oidc_errno = OIDC_EBIND;
     return OIDC_EBIND;
   }
+#ifndef ANY_MSYS
+  if (check_socket_path(con->server->sun_path, group) != OIDC_SUCCESS) {
+    return oidc_errno;
+  }
+#endif
+  umask(previous_mask);
   int flags;
   if (-1 == (flags = fcntl(*(con->sock), F_GETFL, 0)))
     flags = 0;
