@@ -12,8 +12,8 @@
 #include "utils/oidc_error.h"
 #include "utils/string/stringUtils.h"
 
-char* _getIssuerConfig(struct oidc_account* account) {
-  char* cert_path = account_getCertPath(account);
+char* _obtainIssuerConfig(struct oidc_account* account) {
+  char* cert_path = account_getCertPathOrDefault(account);
   char* res       = NULL;
   if (strValid(account_getConfigEndpoint(account))) {
     res = httpsGET(account_getConfigEndpoint(account), NULL, cert_path);
@@ -44,19 +44,20 @@ char* _getIssuerConfig(struct oidc_account* account) {
     }
     account_setConfigEndpoint(account, configuration_endpoint);
   }
+  secFree(cert_path);
   agent_log(DEBUG, "Configuration endpoint is: %s",
             account_getConfigEndpoint(account));
   return res;
 }
-/** @fn oidc_error_t getIssuerConfig(struct oidc_account* account)
+/** @fn oidc_error_t obtainIssuerConfig(struct oidc_account* account)
  * @brief retrieves issuer config from the configuration_endpoint
  * @note the issuer url has to be set prior
  * @param account the account struct, will be updated with the retrieved
  * config
  * @return a oidc_error status code
  */
-oidc_error_t getIssuerConfig(struct oidc_account* account) {
-  char* res = _getIssuerConfig(account);
+oidc_error_t obtainIssuerConfig(struct oidc_account* account) {
+  char* res = _obtainIssuerConfig(account);
   if (NULL == res) {
     return oidc_errno;
   }
@@ -70,10 +71,8 @@ char* getScopesSupportedFor(const char* issuer_url, const char* config_endpoint,
   account_setConfigEndpoint(&account, oidc_strcopy(config_endpoint));
   if (strValid(cert_path)) {
     account_setCertPath(&account, oidc_strcopy(cert_path));
-  } else {
-    account_setOSDefaultCertPath(&account);
   }
-  oidc_error_t err = getIssuerConfig(&account);
+  oidc_error_t err = obtainIssuerConfig(&account);
   if (err != OIDC_SUCCESS) {
     return NULL;
   }
@@ -90,10 +89,8 @@ char* getProvidersSupportedByMytoken(const char* issuer_url,
   account_setConfigEndpoint(&account, oidc_strcopy(config_endpoint));
   if (strValid(cert_path)) {
     account_setCertPath(&account, oidc_strcopy(cert_path));
-  } else {
-    account_setOSDefaultCertPath(&account);
   }
-  char* res = _getIssuerConfig(&account);
+  char* res = _obtainIssuerConfig(&account);
   if (res == NULL) {
     return NULL;
   }
