@@ -1,13 +1,17 @@
 #define _XOPEN_SOURCE 500
+#define _POSIX_C_SOURCE 200112L
 
 #include "oidcp.h"
 
 #include <libgen.h>
 #include <signal.h>
+#include <stdlib.h>
 #include <string.h>
 #include <time.h>
 #include <unistd.h>
 
+#include "account/account.h"
+#include "account/issuer_helper.h"
 #include "config_updater.h"
 #include "defines/ipc_values.h"
 #include "defines/oidc_values.h"
@@ -25,7 +29,11 @@
 #include "oidc-agent/oidcp/passwords/password_store.h"
 #include "oidc-agent/oidcp/proxy_handler.h"
 #include "oidc-agent/oidcp/start_oidcd.h"
+#include "oidc-agent/stats/statlogger.h"
+#include "oidc-gen/promptAndSet/name.h"
 #include "utils/agentLogger.h"
+#include "utils/config/gen_config.h"
+#include "utils/config/issuerConfig.h"
 #include "utils/crypt/crypt.h"
 #include "utils/db/connection_db.h"
 #include "utils/disableTracing.h"
@@ -35,13 +43,100 @@
 #include "utils/oidc/device.h"
 #include "utils/printer.h"
 #include "utils/printerUtils.h"
+#include "utils/prompting/getprompt.h"
+#include "utils/prompting/prompt_mode.h"
 #include "utils/string/stringUtils.h"
 #include "utils/uriUtils.h"
 #ifdef __MSYS__
 #include "utils/registryConnector.h"
 #endif
 
-struct connection* unix_listencon;
+static void handleAccountInfo(struct ipcPipe pipes, int sock) {
+  char* loaded_res = ipc_communicateThroughPipe(pipes, REQUEST_LOADEDACCOUNTS);
+  char* info       = parseForInfo(loaded_res);
+  list_t* loaded   = JSONArrayStringToList(info);
+  secFree(info);
+  char* accountsInfo = getAccountInfos(loaded);
+  secFreeList(loaded);
+  server_ipc_write(sock, RESPONSE_SUCCESS_INFO_OBJECT, accountsInfo);
+  secFree(accountsInfo);
+}
+
+static struct connection* unix_listencon;
+
+static pid_t parent_pid = -1;
+
+static void check_parent_alive(void) {
+  if (parent_pid != -1 && getppid() != parent_pid) {
+    exit(EXIT_SUCCESS);
+  }
+}
+
+_Noreturn static void handleClientComm(struct ipcPipe          pipes,
+                                       const struct arguments* arguments,
+                                       time_t parent_alive_interval) {
+  connectionDB_new();
+  connectionDB_setFreeFunction((void (*)(void*)) & _secFreeConnection);
+  connectionDB_setMatchFunction((matchFunction)connection_comparator);
+
+  time_t deadline = 0;
+  while (1) {
+    deadline = getMinPasswordDeath();
+    if (parent_alive_interval > 0) {
+      time_t parent_check = time(NULL) + parent_alive_interval;
+      if (deadline == 0 || deadline > parent_check) {
+        deadline = parent_check;
+      }
+    }
+    struct connection* con = ipc_readAsyncFromMultipleConnectionsWithTimeout(
+        *unix_listencon, deadline);
+    if (con == NULL) {  // timeout reached
+      if (parent_alive_interval != 0) {
+        check_parent_alive();
+      }
+      removeDeathPasswords();
+      continue;
+    }
+    char* client_req = server_ipc_read(*(con->msgsock));
+    if (client_req == NULL) {
+      server_ipc_writeOidcErrnoPlain(*(con->msgsock));
+    } else {
+      statlog(client_req);
+      INIT_KEY_VALUE(IPC_KEY_REQUEST, IPC_KEY_PASSWORDENTRY, IPC_KEY_SHORTNAME);
+      if (CALL_GETJSONVALUES(client_req) < 0) {
+        server_ipc_write(*(con->msgsock), RESPONSE_BADREQUEST, oidc_serror());
+      } else {
+        KEY_VALUE_VARS(request, passwordentry, shortname);
+        if (_request) {
+          unsigned char skipOIDCDComm = 0;
+          if (strequal(_request, REQUEST_VALUE_ADD) ||
+              strequal(_request, REQUEST_VALUE_GEN)) {
+            pw_handleSave(_passwordentry);
+          } else if (strequal(_request, REQUEST_VALUE_REMOVE)) {
+            removePasswordFor(_shortname);
+          } else if (strequal(_request, REQUEST_VALUE_REMOVEALL)) {
+            removeAllPasswords();
+          } else if (strequal(_request, REQUEST_VALUE_ACCOUNTINFO)) {
+            handleAccountInfo(pipes, *(con->msgsock));
+            skipOIDCDComm = 1;
+          }
+          if (!skipOIDCDComm) {
+            handleOidcdComm(pipes, *(con->msgsock), client_req, arguments);
+          }
+        } else {  //  no request type
+          server_ipc_write(*(con->msgsock), RESPONSE_BADREQUEST,
+                           "No request type.");
+        }
+      }
+      SEC_FREE_KEY_VALUES();
+      secFree(client_req);
+    }
+    agent_log(DEBUG, "Remove con from pool");
+    connectionDB_removeIfFound(con);
+    agent_log(DEBUG, "Currently there are %lu connections",
+              connectionDB_getSize());
+  }
+}
 
 int main(int argc, char** argv) {
   platform_disable_tracing();
@@ -118,14 +213,35 @@ int main(int argc, char** argv) {
   signal(SIGPIPE, SIG_IGN);
   if (ipc_server_init(unix_listencon, arguments.group, arguments.socket_path) !=
       OIDC_SUCCESS) {
-    printError("%s\n", oidc_serror());
+    oidc_perror();
     exit(EXIT_FAILURE);
   }
 
+  time_t parent_alive_interval = 0;
   if (!arguments.console) {
-    pid_t daemon_pid = daemonize();
+    unsigned char commandSet = strValid(arguments.command);
+    if (commandSet) {
+      parent_alive_interval = 10;
+    }
+    pid_t daemon_pid = daemonize(!commandSet);
     if (daemon_pid > 0) {
-// Export PID of new daemon
+      // Export PID of new daemon
+      if (commandSet) {
+        char* pid_str = oidc_sprintf("%d", daemon_pid);
+        if (setenv(OIDC_SOCK_ENV_NAME, unix_listencon->server->sun_path, 1) ==
+                -1 ||
+            setenv(OIDC_PID_ENV_NAME, pid_str, 1) == -1) {
+          secFree(pid_str);
+          oidc_setErrnoError();
+          oidc_perror();
+          exit(oidc_errno);
+        }
+        secFree(pid_str);
+        execvp(arguments.command, arguments.args);
+        oidc_setErrnoError();
+        oidc_perror();
+        exit(oidc_errno);
+      }
 #ifdef __MSYS__
       char daemon_pid_string[12];
       sprintf(daemon_pid_string, "%d", daemon_pid);
@@ -158,63 +274,17 @@ int main(int argc, char** argv) {
 #endif
   }
 
+  parent_pid = getppid();
+
   agent_state.defaultTimeout = arguments.lifetime;
   struct ipcPipe pipes       = startOidcd(&arguments);
 
-  if (ipc_bindAndListen(unix_listencon) != 0) {
+  if (ipc_bindAndListen(unix_listencon, arguments.group) != 0) {
     exit(EXIT_FAILURE);
   }
 
-  handleClientComm(pipes, &arguments);
-}
-
-_Noreturn void handleClientComm(struct ipcPipe          pipes,
-                                const struct arguments* arguments) {
-  connectionDB_new();
-  connectionDB_setFreeFunction((void (*)(void*)) & _secFreeConnection);
-  connectionDB_setMatchFunction((matchFunction)connection_comparator);
-
-  time_t minDeath = 0;
-  while (1) {
-    minDeath               = getMinPasswordDeath();
-    struct connection* con = ipc_readAsyncFromMultipleConnectionsWithTimeout(
-        *unix_listencon, minDeath);
-    if (con == NULL) {  // timeout reached
-      removeDeathPasswords();
-      continue;
-    }
-    char* client_req = server_ipc_read(*(con->msgsock));
-    if (client_req == NULL) {
-      server_ipc_writeOidcErrnoPlain(*(con->msgsock));
-    } else {  // NULL != q
-      INIT_KEY_VALUE(IPC_KEY_REQUEST, IPC_KEY_PASSWORDENTRY, IPC_KEY_SHORTNAME);
-      if (CALL_GETJSONVALUES(client_req) < 0) {
-        server_ipc_write(*(con->msgsock), RESPONSE_BADREQUEST, oidc_serror());
-      } else {
-        KEY_VALUE_VARS(request, passwordentry, shortname);
-        if (_request) {
-          if (strequal(_request, REQUEST_VALUE_ADD) ||
-              strequal(_request, REQUEST_VALUE_GEN)) {
-            pw_handleSave(_passwordentry, arguments->pw_lifetime);
-          } else if (strequal(_request, REQUEST_VALUE_REMOVE)) {
-            removePasswordFor(_shortname);
-          } else if (strequal(_request, REQUEST_VALUE_REMOVEALL)) {
-            removeAllPasswords();
-          }
-          handleOidcdComm(pipes, *(con->msgsock), client_req, arguments);
-        } else {  //  no request type
-          server_ipc_write(*(con->msgsock), RESPONSE_BADREQUEST,
-                           "No request type.");
-        }
-      }
-      SEC_FREE_KEY_VALUES();
-      secFree(client_req);
-    }
-    agent_log(DEBUG, "Remove con from pool");
-    connectionDB_removeIfFound(con);
-    agent_log(DEBUG, "Currently there are %lu connections",
-              connectionDB_getSize());
-  }
+  set_prompt_mode(PROMPT_MODE_GUI);
+  handleClientComm(pipes, &arguments, parent_alive_interval);
 }
 
 char* _extractShortnameFromReauthenticateInfo(const char* info) {
@@ -274,9 +344,12 @@ int _waitForCodeExchangeRequest(time_t expiration, const char* expected_state,
     if (!strcaseequal(_request, REQUEST_VALUE_CODEEXCHANGE)) {
       secFree(client_req);
       SEC_FREE_KEY_VALUES();
-      server_ipc_write(
-          *(con->msgsock), RESPONSE_ERROR,
-          "request currently not acceptable; please try again later");
+      time_t remaining_time = expiration - time(NULL);
+      char*  error_msg      = oidc_sprintf(
+          "request currently not acceptable; please try again later (%ds)",
+          remaining_time);
+      server_ipc_write(*(con->msgsock), RESPONSE_ERROR, error_msg);
+      secFree(error_msg);
       connectionDB_removeIfFound(con);
       agent_log(DEBUG, "Currently there are %lu connections",
                 connectionDB_getSize());
@@ -307,7 +380,7 @@ int _waitForCodeExchangeRequest(time_t expiration, const char* expected_state,
       connectionDB_removeIfFound(con);
       agent_log(DEBUG, "Currently there are %lu connections",
                 connectionDB_getSize());
-      agent_log(DEBUG, "Reaturning");
+      agent_log(DEBUG, "Returning");
       return 0;
     }
     secFree(state);
@@ -319,6 +392,226 @@ int _waitForCodeExchangeRequest(time_t expiration, const char* expected_state,
   }
 }
 
+const char* _getMytokenURLToUse(struct ipcPipe pipes, const char* issuer) {
+  unsigned char useMytokenServer = 0;
+  if (!getGenConfig()->prefer_mytoken_over_oidc ||
+      getGenConfig()->default_mytoken_server == NULL) {
+    return NULL;
+  }
+  char* providers_res = ipc_communicateThroughPipe(
+      pipes, REQUEST_MYTOKEN_PROVIDERS, getGenConfig()->default_mytoken_server,
+      "", "");
+  if (providers_res == NULL) {
+    return NULL;
+  }
+  char* providers = getJSONValueFromString(providers_res, IPC_KEY_INFO);
+  secFree(providers_res);
+  if (providers == NULL) {
+    return NULL;
+  }
+  list_t* providers_l = JSONArrayStringToList(providers);
+  secFree(providers);
+  if (providers_l == NULL) {
+    return NULL;
+  }
+  list_node_t*     node;
+  list_iterator_t* it = list_iterator_new(providers_l, LIST_HEAD);
+  while ((node = list_iterator_next(it))) {
+    char* p   = node->val;
+    char* iss = getJSONValueFromString(p, OIDC_KEY_ISSUER);
+    if (compIssuerUrls(issuer, iss)) {
+      useMytokenServer = 1;
+      break;
+    }
+  }
+  list_iterator_destroy(it);
+  secFreeList(providers_l);
+  return useMytokenServer ? getGenConfig()->default_mytoken_server : NULL;
+}
+
+void _parseInternalGen(struct ipcPipe pipes, int sock, char* res,
+                       const char* original_client_req,
+                       const char* error_res_fmt, const char* error_res_arg,
+                       const char* shortname, unsigned char reauth_intro) {
+  SHUTDOWN_IF_D_DIED(res);
+  INIT_KEY_VALUE(IPC_KEY_DEVICE, IPC_KEY_URI, OIDC_KEY_STATE, IPC_KEY_REQUEST,
+                 INT_IPC_KEY_ACTION, IPC_KEY_ISSUERURL, IPC_KEY_SHORTNAME);
+  if (CALL_GETJSONVALUES(res) < 0) {
+    server_ipc_write(sock, error_res_fmt, error_res_arg);
+    secFree(res);
+    SEC_FREE_KEY_VALUES();
+    return;
+  }
+  secFree(res);
+  KEY_VALUE_VARS(device, url, state, request, action, issuer, shortname);
+  if (_url) {
+    agent_displayAuthCodeURL(_url, shortname, reauth_intro);
+    time_t timeout = time(NULL) + AGENT_PROMPT_TIMEOUT;
+    if (_waitForCodeExchangeRequest(timeout, _state, pipes)) {
+      server_ipc_write(sock, error_res_fmt, error_res_arg);
+      SEC_FREE_KEY_VALUES();
+      return;
+    }
+
+    char* lookup_res =
+        ipc_communicateThroughPipe(pipes, REQUEST_STATELOOKUP, _state);
+    SHUTDOWN_IF_D_DIED(lookup_res);
+    char* config = parseStateLookupRes(lookup_res, pipes);
+    if (config == NULL) {
+      server_ipc_write(sock, error_res_fmt, error_res_arg);
+      SEC_FREE_KEY_VALUES();
+      return;
+    }
+    SEC_FREE_KEY_VALUES();
+    if (writeOIDCFile(config, shortname) != OIDC_SUCCESS) {
+      server_ipc_write(sock, error_res_fmt, error_res_arg);
+      secFree(config);
+      return;
+    }
+    secFree(config);
+
+    char* final_res =
+        ipc_communicateThroughPipe(pipes, "%s", original_client_req);
+    server_ipc_write(sock, "%s", final_res);
+    secFree(final_res);
+    return;
+  }
+  if (_device) {
+    struct oidc_device_code* dc = getDeviceCodeFromJSON(_device);
+    if (dc == NULL) {
+      SEC_FREE_KEY_VALUES();
+      server_ipc_write(sock, error_res_fmt, error_res_arg);
+      return;
+    }
+    agent_displayDeviceCode(dc, shortname, reauth_intro);
+    if (dc->expires_in > AGENT_PROMPT_TIMEOUT) {
+      dc->expires_in = AGENT_PROMPT_TIMEOUT;
+    }
+    char* config = agent_pollDeviceCode(
+        _device, dc->interval, dc->expires_in ? time(NULL) + dc->expires_in : 0,
+        0, &pipes);
+    SEC_FREE_KEY_VALUES();
+    if (config == NULL) {
+      server_ipc_write(sock, error_res_fmt, error_res_arg);
+      return;
+    }
+    if (writeOIDCFile(config, shortname) != OIDC_SUCCESS) {
+      secFree(config);
+      server_ipc_write(sock, error_res_fmt, error_res_arg);
+      return;
+    }
+    secFree(config);
+
+    char* final_res =
+        ipc_communicateThroughPipe(pipes, "%s", original_client_req);
+    server_ipc_write(sock, "%s", final_res);
+    secFree(final_res);
+    return;
+  }
+  if (strcaseequal(_request, INT_REQUEST_VALUE_UPD_ISSUER)) {
+    oidcp_updateIssuerConfig(_action, _issuer, _shortname);
+    SEC_FREE_KEY_VALUES();
+    return _parseInternalGen(
+        pipes, sock, ipc_communicateThroughPipe(pipes, RESPONSE_SUCCESS),
+        original_client_req, error_res_fmt, error_res_arg, shortname,
+        reauth_intro);
+  }
+  SEC_FREE_KEY_VALUES();
+  server_ipc_write(sock, error_res_fmt, error_res_arg);
+}
+
+void handleAutoGen(struct ipcPipe pipes, int sock,
+                   const char* original_client_req, const char* issuer,
+                   const char* scopes, const char* application_hint) {
+  struct oidc_account* account = secAlloc(sizeof(struct oidc_account));
+  set_prompt_mode(PROMPT_MODE_GUI);
+  set_pw_prompt_mode(PROMPT_MODE_GUI);
+  account_setIssuerUrl(account, oidc_strcopy(issuer));
+  account_setMytokenUrl(account,
+                        oidc_strcopy(_getMytokenURLToUse(pipes, issuer)));
+  const list_t* flows          = NULL;
+  unsigned char usedUserClient = 0;
+  if (account_getMytokenUrl(account)) {
+    if (getGenConfig()->default_mytoken_profile) {
+      account_setUsedMytokenProfile(
+          account,
+          oidc_sprintf("\"%s\"", getGenConfig()->default_mytoken_profile));
+    }
+  } else {
+    const struct issuerConfig* issConfig = getIssuerConfig(issuer);
+    if (issConfig && issConfig->user_client &&
+        issConfig->user_client->client_id) {
+      usedUserClient = 1;
+      updateAccountWithUserClientInfo(account);
+      flows = getUserClientFlows(account_getIssuerUrl(account));
+    } else {
+      updateAccountWithPublicClientInfo(account);
+      flows = getPubClientFlows(account_getIssuerUrl(account));
+    }
+    if (!strValid(account_getClientId(account))) {
+      secFreeAccount(account);
+      server_ipc_write(sock, RESPONSE_ERROR, ACCOUNT_NOT_LOADED);
+      return;
+    }
+  }
+  if (strequal(scopes, AGENT_SCOPE_ALL)) {
+    if (account_getMytokenUrl(account)) {
+      account_setScopeExact(account, oidc_strcopy(scopes));
+    } else {
+      account_setScope(account, usedUserClient
+                                    ? getScopesForUserClient(account)
+                                    : getScopesForPublicClient(account));
+    }
+  } else {
+    account_setScope(account, oidc_strcopy(scopes));
+  }
+
+  agent_log(DEBUG, "Prompting user for confirmation for autogen for '%s'",
+            issuer);
+  cJSON* data = generateJSONObject("issuer", cJSON_String, issuer, NULL);
+  data        = jsonAddStringValue(data, "application-hint", application_hint);
+  char* prompt_text = getprompt(PROMPTTEMPLATE(LINK_IDENTITY), data);
+  secFreeJson(data);
+  if (!agent_promptConsentDefaultYes(prompt_text)) {
+    secFree(prompt_text);
+    agent_log(DEBUG, "User declined autogen");
+    server_ipc_write(sock, RESPONSE_ERROR, ACCOUNT_NOT_LOADED);
+    return;
+  }
+  secFree(prompt_text);
+
+  char* name_suggestion = getTopHost(issuer);
+  signal(SIGINT, SIG_IGN);
+  askOrNeedName(account, NULL, NULL, 0, 1, name_suggestion);
+  signal(SIGINT, SIG_DFL);
+  secFree(name_suggestion);
+  if (account_getName(account) == NULL) {  // user canceled prompt
+    secFreeAccount(account);
+    server_ipc_write(sock, RESPONSE_ERROR, ACCOUNT_NOT_LOADED);
+    return;
+  }
+  char* shortname = oidc_strcopy(account_getName(account));
+
+  char* flow = listToJSONArrayString((list_t*)flows);
+  if (flow == NULL) {
+    flow = oidc_strcopy(account_getMytokenUrl(account)
+                            ? "[\"" FLOW_VALUE_MT_OIDC "\"]"
+                            : "[\"" FLOW_VALUE_DEVICE "\",\"" FLOW_VALUE_CODE
+                              "\"]");
+  }
+  char* account_json = accountToJSONString(account);
+  secFreeAccount(account);
+  agent_log(DEBUG, "sending gen request to oidcd");
+  char* gen_res = ipc_communicateThroughPipe(pipes, REQUEST_GEN, account_json,
+                                             flow, "{}", 0, 0, 0);
+  secFree(account_json);
+  secFree(flow);
+  agent_log(DEBUG, "parsing response of gen");
+  _parseInternalGen(pipes, sock, gen_res, original_client_req, RESPONSE_ERROR,
+                    ACCOUNT_NOT_LOADED, shortname, 0);
+  secFree(shortname);
+}
+
 void doReauthenticate(struct ipcPipe pipes, int sock,
                       const char* original_client_req, const char* oidcd_res,
                       const char* info) {
@@ -332,91 +625,9 @@ void doReauthenticate(struct ipcPipe pipes, int sock,
   logger(DEBUG, "Extracted shortname '%s'", shortname);
   char* reauth_res =
       ipc_communicateThroughPipe(pipes, REQUEST_REAUTHENTICATE, shortname);
-  SHUTDOWN_IF_D_DIED(reauth_res);
-  INIT_KEY_VALUE(IPC_KEY_DEVICE, IPC_KEY_URI, OIDC_KEY_STATE);
-  if (CALL_GETJSONVALUES(reauth_res) < 0) {
-    server_ipc_write(sock, "%s", oidcd_res);
-    secFree(reauth_res);
-    SEC_FREE_KEY_VALUES();
-    return;
-  }
-  secFree(reauth_res);
-  KEY_VALUE_VARS(device, url, state);
-  if (_url) {
-    agent_displayAuthCodeURL(_url, shortname);
-    time_t timeout = time(NULL) + AGENT_PROMPT_TIMEOUT;
-    if (_waitForCodeExchangeRequest(timeout, _state, pipes)) {
-      server_ipc_write(sock, "%s", oidcd_res);
-      SEC_FREE_KEY_VALUES();
-      return;
-    }
-
-    char* lookup_res =
-        ipc_communicateThroughPipe(pipes, REQUEST_STATELOOKUP, _state);
-    SHUTDOWN_IF_D_DIED(lookup_res);
-    char* config = parseStateLookupRes(lookup_res);
-    if (config == NULL) {
-      server_ipc_write(sock, "%s", oidcd_res);
-      SEC_FREE_KEY_VALUES();
-      secFree(shortname);
-      return;
-    }
-    SEC_FREE_KEY_VALUES();
-    if (writeOIDCFile(config, shortname) != OIDC_SUCCESS) {
-      server_ipc_write(sock, "%s", oidcd_res);
-      secFree(config);
-      secFree(shortname);
-      return;
-    }
-    secFree(shortname);
-    secFree(config);
-
-    char* final_res =
-        ipc_communicateThroughPipe(pipes, "%s", original_client_req);
-    server_ipc_write(sock, "%s", final_res);
-    secFree(final_res);
-    return;
-  }
-  if (_device) {
-    struct oidc_device_code* dc = getDeviceCodeFromJSON(_device);
-    if (dc == NULL) {
-      SEC_FREE_KEY_VALUES();
-      server_ipc_write(sock, "%s", oidcd_res);
-      secFree(shortname);
-      return;
-    }
-    agent_displayDeviceCode(dc, shortname);
-    if (dc->expires_in > AGENT_PROMPT_TIMEOUT) {
-      dc->expires_in = AGENT_PROMPT_TIMEOUT;
-    }
-    char* config = agent_pollDeviceCode(
-        _device, dc->interval, dc->expires_in ? time(NULL) + dc->expires_in : 0,
-        0, &pipes);
-    SEC_FREE_KEY_VALUES();
-    if (config == NULL) {
-      server_ipc_write(sock, "%s", oidcd_res);
-      secFree(shortname);
-      return;
-    }
-    if (writeOIDCFile(config, shortname) != OIDC_SUCCESS) {
-      server_ipc_write(sock, "%s", oidcd_res);
-      secFree(config);
-      secFree(shortname);
-      return;
-    }
-    secFree(config);
-    secFree(shortname);
-
-    char* final_res =
-        ipc_communicateThroughPipe(pipes, "%s", original_client_req);
-    server_ipc_write(sock, "%s", final_res);
-    secFree(final_res);
-    return;
-  }
-  SEC_FREE_KEY_VALUES();
-  server_ipc_write(sock, "%s", oidcd_res);
+  _parseInternalGen(pipes, sock, reauth_res, original_client_req, "%s",
+                    oidcd_res, shortname, 1);
   secFree(shortname);
-  return;
 }
 
 void handleOidcdComm(struct ipcPipe pipes, int sock, const char* msg,
@@ -424,7 +635,7 @@ void handleOidcdComm(struct ipcPipe pipes, int sock, const char* msg,
   char* send = oidc_strcopy(msg);
   INIT_KEY_VALUE(IPC_KEY_REQUEST, OIDC_KEY_REFRESHTOKEN, IPC_KEY_SHORTNAME,
                  IPC_KEY_APPLICATIONHINT, IPC_KEY_ISSUERURL, OIDC_KEY_ERROR,
-                 IPC_KEY_INFO);
+                 IPC_KEY_INFO, INT_IPC_KEY_ACTION, OIDC_KEY_SCOPE);
   while (1) {
     // RESET_KEY_VALUE_VALUES_TO_NULL();
     char* oidcd_res = ipc_communicateThroughPipe(pipes, "%s", send);
@@ -438,7 +649,7 @@ void handleOidcdComm(struct ipcPipe pipes, int sock, const char* msg,
       return;
     }
     KEY_VALUE_VARS(request, refresh_token, shortname, application_hint, issuer,
-                   error, info);
+                   error, info, action, scope);
     if (_request == NULL) {  // if the response is the final response, forward
                              // it to the client
       if (_error != NULL && _info != NULL &&
@@ -458,6 +669,7 @@ void handleOidcdComm(struct ipcPipe pipes, int sock, const char* msg,
       SEC_FREE_KEY_VALUES();
       return;
     }
+    statlog(oidcd_res);
     secFree(oidcd_res);
     if (strequal(_request, INT_REQUEST_VALUE_UPD_REFRESH)) {
       oidc_error_t e = updateRefreshToken(_shortname, _refresh_token);
@@ -465,6 +677,11 @@ void handleOidcdComm(struct ipcPipe pipes, int sock, const char* msg,
                                          : oidc_sprintf(RESPONSE_ERROR, oidc_serror());
       SEC_FREE_KEY_VALUES();
       continue;
+    } else if (strequal(_request, INT_REQUEST_VALUE_UPD_ISSUER)) {
+      oidcp_updateIssuerConfig(_action, _issuer, _shortname);
+      send = oidc_strcopy(RESPONSE_SUCCESS);
+      SEC_FREE_KEY_VALUES();
+      continue;
     } else if (strequal(_request, INT_REQUEST_VALUE_AUTOLOAD)) {
       char* config = getAutoloadConfig(_shortname, _issuer, _application_hint);
       send         = config
@@ -473,6 +690,10 @@ void handleOidcdComm(struct ipcPipe pipes, int sock, const char* msg,
       secFree(config);
       SEC_FREE_KEY_VALUES();
       continue;
+    } else if (strequal(_request, INT_REQUEST_VALUE_AUTOGEN)) {
+      handleAutoGen(pipes, sock, msg, _issuer, _scope, _application_hint);
+      SEC_FREE_KEY_VALUES();
+      return;
     } else if (strequal(_request, INT_REQUEST_VALUE_CONFIRM)) {
       oidc_error_t e =
           _issuer ? askpass_getConfirmationWithIssuer(_issuer, _shortname,
@@ -499,14 +720,13 @@ void handleOidcdComm(struct ipcPipe pipes, int sock, const char* msg,
       SEC_FREE_KEY_VALUES();
       continue;
     } else if (strequal(_request, INT_REQUEST_VALUE_QUERY_ACCDEFAULT)) {
-      char* account = NULL;
+      const char* account = NULL;
       if (strValid(_issuer)) {  // default for this issuer
         account = getDefaultAccountConfigForIssuer(_issuer);
       } else {                      // global default
         oidc_errno = OIDC_NOTIMPL;  // TODO
       }
       send = oidc_sprintf(INT_RESPONSE_ACCDEFAULT, account ?: "");
-      secFree(account);
       SEC_FREE_KEY_VALUES();
       continue;
     } else {
