@@ -2,7 +2,7 @@
 #define OIDCP_AGENT_PROMPT_H
 
 #include "oidc-agent/oidc/device_code.h"
-#include "utils/prompt.h"
+#include "utils/prompting/prompt.h"
 
 #define AGENT_PROMPT_TIMEOUT PROMPT_DEFAULT_TIMEOUT
 
@@ -11,8 +11,9 @@ char* agent_promptPassword(const char* text, const char* label,
 int   agent_promptConsentDefaultYes(const char* text);
 char* agent_promptMytokenConsent(const char* base64_html);
 
-void agent_displayAuthCodeURL(const char* url, const char* shortname);
+void agent_displayAuthCodeURL(const char* url, const char* shortname,
+                              unsigned char reauth_intro);
 void agent_displayDeviceCode(const struct oidc_device_code* device,
-                             const char*                    shortname);
+                             const char* shortname, unsigned char reauth_intro);
 
 #endif /* OIDCP_AGENT_PROMPT_H */
